//
//  XSFSingletonMacro
//  XSFoundation
//
//  Created by Bertrand Landry-Hétu on 2011/02/16.
//  Copyright 2011 Xsilva Systems. All rights reserved.
//


//XSF_SYNTHESIZE_SHARED_OBJECT_FOR_CLASS generates a unit test friendly "sharedObject"/singleton
//it isn't a singleton in the traditional way (you can still allocate more instances of the object)
//but should be used like one. It also makes it easier to unit test objects that uses the singleton

#if __MAC_OS_X_VERSION_MIN_REQUIRED >= 1060 || __IPHONE_OS_VERSION_MIN_REQUIRED >= 40000
//The 10.6 & iOS 4 version of the macro is thread safe as it was cheap and easy, not the normal one.

#import <dispatch/dispatch.h>

#define XSF_SYNTHESIZE_SHARED_OBJECT_FOR_CLASS(class_name, method_name) \
+ (class_name*)method_name { \
    static dispatch_once_t once; \
    static class_name *shared##class_name; \
    dispatch_once(&once, ^ { shared##class_name = [self alloc]; shared##class_name = [shared##class_name init]; }); \
    return shared##class_name; \
}

#else

#define XSF_SYNTHESIZE_SHARED_OBJECT_FOR_CLASS(class_name, method_name) \
+ (class_name*)method_name { \
	static class_name *shared##class_name; \
    if (!sharedInstance) { \
        shared##class_name = [self alloc]; shared##class_name = [shared##class_name init]; \
    } \
    return shared##class_name;\
}

#endif


#define XSF_SYNTHESIZE_SHARED_OBJECT_FOR_CLASS_HEADER(class_name, method_name) \
+ (class_name*)method_name;


/* Example:
 *
 * KJSomeManager.h:
 * ========================================
 *	#import "XSFSingletonMacro.h"
 *
 *	@interface KJSomeManager: SomeSuperclass
 *	{
 *		int value;
 *		...
 *	}
 *	XSF_SYNTHESIZE_SHARED_OBJECT_FOR_CLASS_HEADER(KJSomeManager, sharedSomeManager);
 *
 *	@end
 * ========================================
 *
 *
 *	KJSomeManager.m:
 * ========================================
 *	#import "KJSomeManager.h"
 *
 *	@implementation KJSomeManager
 *
 *	XSF_SYNTHESIZE_SHARED_OBJECT_FOR_CLASS(KJSomeManager, sharedSomeManager);
 *
 *	...
 *
 *	@end
 * ========================================
 */

